<?php
Zend_Loader::loadClass("Zend_Db");
Zend_Loader::loadClass("Zend_Db_Statement");
Zend_Loader::loadClass("CFW_Data_Connection");
Zend_Loader::loadClass("CFW_Models_Entity");
Zend_Loader::loadClass("CFW_Util_String");
Zend_Loader::loadClass("CFW_Util_Date");
/**
 * データベース実行command
 *
 * <del>PDOを使うので、このオブジェクトが直接PDO_Statementを作成し実行する</del>
 * Zend_Dbを使用するのでこのオブジェクトがZend_Db_Statementを作成して実行する。
 *
 * パラメータについてはクエリー文字列に直接埋め込まず、パラメータクエリを設定してパラメータは別に渡す。セキュリティ対策のみならず単純な単引用符、セミコロンの入力に対する強度を上げるためとクエリーキャッシュを使うため。
 *
 * @author okada
 * @package CFW_Data
 */
class CFW_Data_Command{
    /**
     * クエリー文字列。
     *
     * $q = "select * from hoge where id = ".$id; 等のようにはせず、$q = "select * from hoge where id = ?";等のようにする。
     * @var string
     */
    private $text;
    /**
     * クエリーに与えるパラメータ
     * @var array CFW_Data_Parameterを保持する。
     */
    private $parameters;

    /**
     * このコマンドを実行する接続先
     * @var CFW_Data_Connection
     */
    private $connection;
    /**
     * Zend_Db_Statement 実体
     * @var Zend_Db_Statement
     */
    private $statement;
    
    private $logger;
    private $currentUser;
    /**
     * @param string $text
     * @return unknown_type
     */
    public function __construct($text,$parameters = array()){
        $this->text = $text;
        $this->parameters = $parameters;
    	$this->logger = CFW_Log_LoggerFactory::getLogger("cfw_debug",__CLASS__);   
    }
    /**
     * 対象の接続を設定する
     * @param unknown_type $connection
     * @return unknown_type
     */
    public function setConnection(&$connection){
        $this->connection= $connection;
        $this->currentAuthUser = $connection->currentAuthUser;
    }
    /**
     * パラメータコレクションを取得する。
     * @return unknown_type
     */
    public function getParameters(){
        return $this->parameters;
    }
    /**
     * パラメータを追加する
     * @param unknown_type $parameter
     * @return unknown_type
     */
    public function addParameter($parameter){
        $this->parameters[] = $parameter;
    }

    /**
     * 問い合わせを実行してデータの配列を返す。
     * @return Array 問い合わせ結果の配列(行、列)
     * @see PDOStatement#fetchAll
     */
    public function executeQuery(){
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"start",$this->currentAuthUser);
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,$this,$this->currentAuthUser);
		
		if($this->connection== null){
		    throw new Exception("NO CONNECTION");
		}
		
		$this->prepare();
        $this->statement->execute($this->parametersToArray());
        $result = $this->statement->fetchAll();

		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"end",$this->currentAuthUser);
		return $result;
    }
    /**
     * 問い合わせを実行してデータの配列を返す。
     * @return Array 問い合わせ結果の配列(行、列)
     * @see PDOStatement#fetchAll
     */
    public function executeMultiQuery(){
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"start",$this->currentAuthUser);
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,$this,$this->currentAuthUser);

		if($this->connection == null){
		    throw new Exception("NO CONNECTION");
		}
		
		$this->prepare();
        $this->statement->execute($this->parametersToArray());
        $result = array();
        do{
            $result[] = $this->statement->fetchAll();
        }while($this->statement->nextRowset());

        $this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"end",$this->currentAuthUser);
        return $result;
    }
    /**
     * 問い合わせを実行して単一の値を返す。
     *
     * たとえば "select getdate()"のようなもの。
     * @return object 問い合わせ結果の値。
     * @see PDOStatement#fetchAll
     */
    public function executeScalar(){
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"start",$this->currentAuthUser);
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,$this,$this->currentAuthUser);

		if($this->connection == null){
		    throw new Exception("NO CONNECTION");
		}
		
		$this->prepare();
        $this->statement->execute($this->parametersToArray());
        $result =$this->statement->fetchAll(Zend_Db::FETCH_BOTH);

        $this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"end",$this->currentAuthUser);
        return $result[0][0];
    }
    /**
     * 問い合わせを実行してデータリーダーを作る
     * @return CFW_Data_Reader 問い合わせ結果を列挙できるReader
     */
    public function executeReader(){
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"start",$this->currentAuthUser);
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,$this,$this->currentAuthUser);
    	
		if($this->connection == null){
		    throw new Exception("NO CONNECTION");
		}
		
		$this->prepare();
        $this->statement->execute($this->parametersToArray());
        $reader = new CFW_Data_Reader();
        $reader->connection = $this->connection;
        $reader->statement = $this->statement;

        $this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"end",$this->currentAuthUser);
        return $reader;
    }
    /**
     * 問い合わせを実行して指定クラス名のオブジェクトの配列を返す。
     * @return Array 問い合わせ結果の配列(行、列)
     * @see PDOStatement#fetchAll
     */
    public function executeQueryAsObject($className = "CFW_Models_Entity",$prefix = ""){
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"start",$this->currentAuthUser);
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,$this,$this->currentAuthUser);
    	
		if($this->connection == null){
		    throw new Exception("NO CONNECTION");
		}
		
		Zend_Loader::LoadClass($className);
        $this->prepare();
        $this->statement->execute($this->parametersToArray());

        $result = array();
        while($row = $this->statement->fetch()){
            $object = new $className();
            $object->map($row,$prefix);
            $result[] = $object;
        }
        $this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"end",$this->currentAuthUser);
        return $result;
    }
    /**
     * <del>更新処理を実行する。<del>実際には「結果セットを返さないクエリー全般を処理する。
     * @deprecated 紛らわしいので廃止。
     * @return unknown_type
     */
    public function executeUpdate(){
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"start",$this->currentAuthUser);
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,$this,$this->currentAuthUser);
    	
		if($this->connection == null){
		    throw new Exception("NO CONNECTION");
		}
		
		$this->prepare();
        $result = $this->statement->execute($this->parametersToArray());
        $count = $this->statement->rowCount();
        $this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"end",$this->currentAuthUser);
        return $count;

    }
    /**
     * 結果セットを返さないクエリーを実行する。
     *
     * @return int 影響を受けた件数(SQLServerの場合@@ROWCOUNT)
     */
    public function executeNonQuery(){
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"start",$this->currentAuthUser);
		$this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,$this,$this->currentAuthUser);
    	
		if($this->connection == null){
		    throw new Exception("NO CONNECTION");
		}
		
		$this->prepare();
        $result = $this->statement->execute($this->parametersToArray());
        $count = $this->statement->rowCount();

        $this->logger->log(CFW_Log_Level::DEBUG,__FUNCTION__,"end",$this->currentAuthUser);
        return $count;

    }
    /**
     * @return Zend_Db_Statement
     */
    private function prepare(){
    	/* @var Zend_Db_Adapter_Abstract */
        $connection = &$this->connection->_connection;
        $this->statement = $connection->prepare($this->text);

        return $this->statement ;
    }
    /**
     * パラメータをステートメントオブジェクトにバインド
     * @return unknown_type
     */
    function bindParameters(){
    	$paramOrdinal = 1;
        foreach($this->parameters as $param){
            $this->statement->bindParam($paramOrdinal,$param->value);
        }
    }
    function parametersToArray(){
    	
        $parameters = array();
        foreach($this->parameters as $param){
        	$value = $param->value;
	        if ($value instanceof DateTime){
	        	$value = CFW_Util_Date::toDate($value);;
	        }
	        else if($value instanceof Date){
	        	$value = CFW_Util_Date::toDate($value);;
	        }
	        else if($value instanceof Zend_Date){
	        	$value = CFW_Util_Date::toDate($value);;
	        }

            $parameters[] = $value;
        }

        return $parameters;

    }
    function __toString(){
    	$expression = get_class($this).PHP_EOL."{text : ";
    	$expression .= $this->text;
    	$expression .= ",".PHP_EOL ;
		$params = "";
    	foreach($this->parameters as $parameter){
    		if($params != "") $params.=",".PHP_EOL;
    		$params .= "\"{$parameter->name}\" :".CFW_Util_String::toString($parameter->value);
    	}
		$expression .= "parameters:[{$params}]";
    	$expression .= "}";
		return $expression;
    }


}